@@ -6,6 +6,7 @@ use pubgrub::solver::{Dependencies, DependencyProvider};
 use pep440_rs::Version;
 
 use crate::pubgrub::{PubGrubPackage, PubGrubPriority};
+use crate::resolver::UnavailableReason;
 
 /// We don't use a dependency provider, we interact with state directly, but we still need this one
 /// for type
@@ -15,6 +16,8 @@ impl DependencyProvider for UvDependencyProvider {
     type P = PubGrubPackage;
     type V = Version;
     type VS = Range<Version>;
+    type M = UnavailableReason;
+
     fn prioritize(&self, _package: &Self::P, _range: &Self::VS) -> Self::Priority {
         unimplemented!()
     }
@@ -34,7 +37,7 @@ impl DependencyProvider for UvDependencyProvider {
         &self,
         _package: &Self::P,
         _version: &Self::V,
-    ) -> Result<Dependencies<Vec<(Self::P, Self::VS)>>, Self::Err> {
+    ) -> Result<Dependencies<Vec<(Self::P, Self::VS)>, Self::M>, Self::Err> {
         unimplemented!()
     }
 }
