@@ -3,15 +3,12 @@ use rustc_hash::FxHashSet;
 use pep508_rs::MarkerEnvironment;
 use uv_normalize::PackageName;
 
-use crate::Manifest;
+use crate::{DependencyMode, Manifest};
 
-#[derive(Debug, Default, Clone, Copy, PartialEq, Eq)]
+#[derive(Debug, Default, Clone, Copy, PartialEq, Eq, serde::Deserialize)]
+#[serde(deny_unknown_fields, rename_all = "kebab-case")]
 #[cfg_attr(feature = "clap", derive(clap::ValueEnum))]
-#[cfg_attr(feature = "serde", derive(serde::Deserialize))]
-#[cfg_attr(
-    feature = "serde",
-    serde(deny_unknown_fields, rename_all = "kebab-case")
-)]
+#[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
 pub enum ResolutionMode {
     /// Resolve the highest compatible version of each package.
     #[default]
@@ -41,13 +38,17 @@ impl ResolutionStrategy {
         mode: ResolutionMode,
         manifest: &Manifest,
         markers: &MarkerEnvironment,
+        dependencies: DependencyMode,
     ) -> Self {
         match mode {
             ResolutionMode::Highest => Self::Highest,
             ResolutionMode::Lowest => Self::Lowest,
-            ResolutionMode::LowestDirect => {
-                Self::LowestDirect(manifest.direct_dependencies(markers).cloned().collect())
-            }
+            ResolutionMode::LowestDirect => Self::LowestDirect(
+                manifest
+                    .user_requirements(markers, dependencies)
+                    .cloned()
+                    .collect(),
+            ),
         }
     }
 }
