@@ -9,15 +9,16 @@ use uv_configuration::{
     ConfigSettings, IndexStrategy, KeyringProviderType, NoBinary, NoBuild, PreviewMode, Reinstall,
     SetupPyStrategy, TargetTriple, Upgrade,
 };
+use uv_interpreter::{PythonVersion, Target};
 use uv_normalize::PackageName;
 use uv_requirements::ExtrasSpecification;
 use uv_resolver::{AnnotationStyle, DependencyMode, ExcludeNewer, PreReleaseMode, ResolutionMode};
-use uv_toolchain::PythonVersion;
 use uv_workspace::{PipOptions, Workspace};
 
 use crate::cli::{
-    ColorChoice, GlobalArgs, Maybe, PipCheckArgs, PipCompileArgs, PipFreezeArgs, PipInstallArgs,
-    PipListArgs, PipShowArgs, PipSyncArgs, PipUninstallArgs, RunArgs, VenvArgs,
+    ColorChoice, GlobalArgs, LockArgs, Maybe, PipCheckArgs, PipCompileArgs, PipFreezeArgs,
+    PipInstallArgs, PipListArgs, PipShowArgs, PipSyncArgs, PipUninstallArgs, RunArgs, SyncArgs,
+    VenvArgs,
 };
 use crate::commands::ListFormat;
 
@@ -85,9 +86,7 @@ pub(crate) struct RunSettings {
     // CLI-only settings.
     pub(crate) target: Option<String>,
     pub(crate) args: Vec<OsString>,
-    pub(crate) isolated: bool,
     pub(crate) with: Vec<String>,
-    pub(crate) no_workspace: bool,
     pub(crate) python: Option<String>,
 }
 
@@ -98,9 +97,7 @@ impl RunSettings {
         let RunArgs {
             target,
             args,
-            isolated,
             with,
-            no_workspace,
             python,
         } = args;
 
@@ -108,9 +105,49 @@ impl RunSettings {
             // CLI-only settings.
             target,
             args,
-            isolated,
             with,
-            no_workspace,
+            python,
+        }
+    }
+}
+
+/// The resolved settings to use for a `sync` invocation.
+#[allow(clippy::struct_excessive_bools, dead_code)]
+#[derive(Debug, Clone)]
+pub(crate) struct SyncSettings {
+    // CLI-only settings.
+    pub(crate) python: Option<String>,
+}
+
+impl SyncSettings {
+    /// Resolve the [`SyncSettings`] from the CLI and workspace configuration.
+    #[allow(clippy::needless_pass_by_value)]
+    pub(crate) fn resolve(args: SyncArgs, _workspace: Option<Workspace>) -> Self {
+        let SyncArgs { python } = args;
+
+        Self {
+            // CLI-only settings.
+            python,
+        }
+    }
+}
+
+/// The resolved settings to use for a `lock` invocation.
+#[allow(clippy::struct_excessive_bools, dead_code)]
+#[derive(Debug, Clone)]
+pub(crate) struct LockSettings {
+    // CLI-only settings.
+    pub(crate) python: Option<String>,
+}
+
+impl LockSettings {
+    /// Resolve the [`LockSettings`] from the CLI and workspace configuration.
+    #[allow(clippy::needless_pass_by_value)]
+    pub(crate) fn resolve(args: LockArgs, _workspace: Option<Workspace>) -> Self {
+        let LockArgs { python } = args;
+
+        Self {
+            // CLI-only settings.
             python,
         }
     }
@@ -126,6 +163,7 @@ pub(crate) struct PipCompileSettings {
     pub(crate) r#override: Vec<PathBuf>,
     pub(crate) refresh: Refresh,
     pub(crate) upgrade: Upgrade,
+    pub(crate) uv_lock: bool,
 
     // Shared settings.
     pub(crate) shared: PipSharedSettings,
@@ -158,6 +196,7 @@ impl PipCompileSettings {
             offline,
             no_offline,
             refresh,
+            no_refresh,
             refresh_package,
             link_mode,
             index_url,
@@ -170,6 +209,7 @@ impl PipCompileSettings {
             system,
             no_system,
             upgrade,
+            no_upgrade,
             upgrade_package,
             generate_hashes,
             no_generate_hashes,
@@ -193,6 +233,8 @@ impl PipCompileSettings {
             no_emit_marker_expression,
             emit_index_annotation,
             no_emit_index_annotation,
+            unstable_uv_lock_file,
+            no_unstable_uv_lock_file,
             compat_args: _,
         } = args;
 
@@ -204,8 +246,9 @@ impl PipCompileSettings {
                 .filter_map(Maybe::into_option)
                 .collect(),
             r#override,
-            refresh: Refresh::from_args(refresh, refresh_package),
-            upgrade: Upgrade::from_args(upgrade, upgrade_package),
+            refresh: Refresh::from_args(flag(refresh, no_refresh), refresh_package),
+            upgrade: Upgrade::from_args(flag(upgrade, no_upgrade), upgrade_package),
+            uv_lock: flag(unstable_uv_lock_file, no_unstable_uv_lock_file).unwrap_or(false),
 
             // Shared settings.
             shared: PipSharedSettings::combine(
@@ -283,10 +326,12 @@ impl PipSyncSettings {
         let PipSyncArgs {
             src_file,
             reinstall,
+            no_reinstall,
             reinstall_package,
             offline,
             refresh,
             no_offline,
+            no_refresh,
             refresh_package,
             link_mode,
             index_url,
@@ -302,6 +347,7 @@ impl PipSyncSettings {
             no_system,
             break_system_packages,
             no_break_system_packages,
+            target,
             legacy_setup_py,
             no_legacy_setup_py,
             no_build_isolation,
@@ -323,8 +369,8 @@ impl PipSyncSettings {
         Self {
             // CLI-only settings.
             src_file,
-            reinstall: Reinstall::from_args(reinstall, reinstall_package),
-            refresh: Refresh::from_args(refresh, refresh_package),
+            reinstall: Reinstall::from_args(flag(reinstall, no_reinstall), reinstall_package),
+            refresh: Refresh::from_args(flag(refresh, no_refresh), refresh_package),
 
             // Shared settings.
             shared: PipSharedSettings::combine(
@@ -332,6 +378,7 @@ impl PipSyncSettings {
                     python,
                     system: flag(system, no_system),
                     break_system_packages: flag(break_system_packages, no_break_system_packages),
+                    target,
                     offline: flag(offline, no_offline),
                     index_url: index_url.and_then(Maybe::into_option),
                     extra_index_url: extra_index_url.map(|extra_index_urls| {
@@ -380,6 +427,7 @@ pub(crate) struct PipInstallSettings {
     pub(crate) reinstall: Reinstall,
     pub(crate) refresh: Refresh,
     pub(crate) dry_run: bool,
+    pub(crate) uv_lock: Option<String>,
     // Shared settings.
     pub(crate) shared: PipSharedSettings,
 }
@@ -397,12 +445,15 @@ impl PipInstallSettings {
             all_extras,
             no_all_extras,
             upgrade,
+            no_upgrade,
             upgrade_package,
             reinstall,
+            no_reinstall,
             reinstall_package,
             offline,
             refresh,
             no_offline,
+            no_refresh,
             refresh_package,
             no_deps,
             deps,
@@ -423,6 +474,7 @@ impl PipInstallSettings {
             no_system,
             break_system_packages,
             no_break_system_packages,
+            target,
             legacy_setup_py,
             no_legacy_setup_py,
             no_build_isolation,
@@ -440,6 +492,8 @@ impl PipInstallSettings {
             no_strict,
             exclude_newer,
             dry_run,
+            unstable_uv_lock_file,
+            compat_args: _,
         } = args;
 
         Self {
@@ -452,10 +506,11 @@ impl PipInstallSettings {
                 .filter_map(Maybe::into_option)
                 .collect(),
             r#override,
-            upgrade: Upgrade::from_args(upgrade, upgrade_package),
-            reinstall: Reinstall::from_args(reinstall, reinstall_package),
-            refresh: Refresh::from_args(refresh, refresh_package),
+            upgrade: Upgrade::from_args(flag(upgrade, no_upgrade), upgrade_package),
+            reinstall: Reinstall::from_args(flag(reinstall, no_reinstall), reinstall_package),
+            refresh: Refresh::from_args(flag(refresh, no_refresh), refresh_package),
             dry_run,
+            uv_lock: unstable_uv_lock_file,
 
             // Shared settings.
             shared: PipSharedSettings::combine(
@@ -463,6 +518,7 @@ impl PipInstallSettings {
                     python,
                     system: flag(system, no_system),
                     break_system_packages: flag(break_system_packages, no_break_system_packages),
+                    target,
                     offline: flag(offline, no_offline),
                     index_url: index_url.and_then(Maybe::into_option),
                     extra_index_url: extra_index_url.map(|extra_index_urls| {
@@ -530,6 +586,7 @@ impl PipUninstallSettings {
             no_system,
             break_system_packages,
             no_break_system_packages,
+            target,
             offline,
             no_offline,
         } = args;
@@ -545,6 +602,7 @@ impl PipUninstallSettings {
                     python,
                     system: flag(system, no_system),
                     break_system_packages: flag(break_system_packages, no_break_system_packages),
+                    target,
                     offline: flag(offline, no_offline),
                     keyring_provider,
                     ..PipOptions::default()
@@ -726,6 +784,7 @@ impl PipCheckSettings {
 pub(crate) struct VenvSettings {
     // CLI-only settings.
     pub(crate) seed: bool,
+    pub(crate) allow_existing: bool,
     pub(crate) name: PathBuf,
     pub(crate) prompt: Option<String>,
     pub(crate) system_site_packages: bool,
@@ -742,6 +801,7 @@ impl VenvSettings {
             system,
             no_system,
             seed,
+            allow_existing,
             name,
             prompt,
             system_site_packages,
@@ -760,6 +820,7 @@ impl VenvSettings {
         Self {
             // CLI-only settings.
             seed,
+            allow_existing,
             name,
             prompt,
             system_site_packages,
@@ -801,6 +862,7 @@ pub(crate) struct PipSharedSettings {
     pub(crate) system: bool,
     pub(crate) extras: ExtrasSpecification,
     pub(crate) break_system_packages: bool,
+    pub(crate) target: Option<Target>,
     pub(crate) connectivity: Connectivity,
     pub(crate) index_strategy: IndexStrategy,
     pub(crate) keyring_provider: KeyringProviderType,
@@ -840,6 +902,7 @@ impl PipSharedSettings {
             python,
             system,
             break_system_packages,
+            target,
             offline,
             index_url,
             extra_index_url,
@@ -955,6 +1018,7 @@ impl PipSharedSettings {
                 .break_system_packages
                 .or(break_system_packages)
                 .unwrap_or_default(),
+            target: args.target.or(target).map(Target::from),
             no_binary: NoBinary::from_args(args.no_binary.or(no_binary).unwrap_or_default()),
             compile_bytecode: args
                 .compile_bytecode
